@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2007 Jeffrey Stedfast
+ *  Copyright (C) 2000-2008 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -38,28 +38,39 @@ G_BEGIN_DECLS
 typedef struct _GMimeMultipart GMimeMultipart;
 typedef struct _GMimeMultipartClass GMimeMultipartClass;
 
+/**
+ * GMimeMultipart:
+ * @parent_object: parent #GMimeObject
+ * @children: array of MIME sub-parts
+ * @boundary: MIME boundary
+ * @preface: multipart preface
+ * @postface: multipart postface
+ *
+ * A base MIME multipart object.
+ **/
 struct _GMimeMultipart {
 	GMimeObject parent_object;
 	
+	GPtrArray *children;
 	char *boundary;
-	
 	char *preface;
 	char *postface;
-	
-	GList *subparts;
 };
 
 struct _GMimeMultipartClass {
 	GMimeObjectClass parent_class;
 	
-	void (* add_part) (GMimeMultipart *multipart, GMimeObject *part);
-	void (* add_part_at) (GMimeMultipart *multipart, GMimeObject *part, int index);
-	void (* remove_part) (GMimeMultipart *multipart, GMimeObject *part);
-	
-	GMimeObject * (* remove_part_at) (GMimeMultipart *multipart, int index);
+	void (* clear) (GMimeMultipart *multipart);
+	void (* add) (GMimeMultipart *multipart, GMimeObject *part);
+	void (* insert) (GMimeMultipart *multipart, int index, GMimeObject *part);
+	gboolean (* remove) (GMimeMultipart *multipart, GMimeObject *part);
+	GMimeObject * (* remove_at) (GMimeMultipart *multipart, int index);
 	GMimeObject * (* get_part) (GMimeMultipart *multipart, int index);
 	
-	int  (* get_number) (GMimeMultipart *multipart);
+	gboolean (* contains) (GMimeMultipart *multipart, GMimeObject *part);
+	int (* index_of) (GMimeMultipart *multipart, GMimeObject *part);
+	
+	int  (* get_count) (GMimeMultipart *multipart);
 	
 	void (* set_boundary) (GMimeMultipart *multipart, const char *boundary);
 	const char * (* get_boundary) (GMimeMultipart *multipart);
@@ -78,14 +89,18 @@ const char *g_mime_multipart_get_preface (GMimeMultipart *multipart);
 void g_mime_multipart_set_postface (GMimeMultipart *multipart, const char *postface);
 const char *g_mime_multipart_get_postface (GMimeMultipart *multipart);
 
-void g_mime_multipart_add_part (GMimeMultipart *multipart, GMimeObject *part);
-void g_mime_multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index);
-void g_mime_multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part);
+void g_mime_multipart_clear (GMimeMultipart *multipart);
 
-GMimeObject *g_mime_multipart_remove_part_at (GMimeMultipart *multipart, int index);
+void g_mime_multipart_add (GMimeMultipart *multipart, GMimeObject *part);
+void g_mime_multipart_insert (GMimeMultipart *multipart, int index, GMimeObject *part);
+gboolean g_mime_multipart_remove (GMimeMultipart *multipart, GMimeObject *part);
+GMimeObject *g_mime_multipart_remove_at (GMimeMultipart *multipart, int index);
 GMimeObject *g_mime_multipart_get_part (GMimeMultipart *multipart, int index);
 
-int g_mime_multipart_get_number (GMimeMultipart *multipart);
+gboolean g_mime_multipart_contains (GMimeMultipart *multipart, GMimeObject *part);
+int g_mime_multipart_index_of (GMimeMultipart *multipart, GMimeObject *part);
+
+int g_mime_multipart_get_count (GMimeMultipart *multipart);
 
 void g_mime_multipart_set_boundary (GMimeMultipart *multipart, const char *boundary);
 const char *g_mime_multipart_get_boundary (GMimeMultipart *multipart);
