@@ -74,11 +74,10 @@ print_mime_struct (GMimeStream *stream, GMimeObject *part, int depth)
 	if (GMIME_IS_MULTIPART (part)) {
 		multipart = (GMimeMultipart *) part;
 		
-		n = g_mime_multipart_get_number (multipart);
+		n = g_mime_multipart_get_count (multipart);
 		for (i = 0; i < n; i++) {
 			subpart = g_mime_multipart_get_part (multipart, i);
 			print_mime_struct (stream, subpart, depth + 1);
-			g_object_unref (subpart);
 		}
 	} else if (GMIME_IS_MESSAGE_PART (part)) {
 		mpart = (GMimeMessagePart *) part;
@@ -420,6 +419,7 @@ int main (int argc, char **argv)
 		
 		g_object_unref (istream);
 		g_object_unref (ostream);
+		g_object_unref (parser);
 		
 #ifdef ENABLE_MBOX_MATCH
 		g_object_unref (mstream);
